package com.zayac.changeurl.service;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ReUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.system.SystemUtil;
import com.zayac.changeurl.entity.Msg2JSEntity;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zayac
 * @since 2023-08-30 12:08
 */
@Service
public class ChangeURLServiceImpl implements ChangeURLService {

    public static final String URL_PATTERN = "(https?://[a-zA-Z0-9.-]+\\.[a-zA-Z]{2,6})(:[0-9]{1,5})?";
    private static final String PREFER = "新";
    @Value("${js.path.targets}")
    //开云唯一词
    private String[] targets;

    @Value("${js.path.template}")
    //开云唯一词
    private String templatePath;

    @Value("${msg.ky.startWith.web}")
    private String msgKyWeb;

    @Value("${msg.ky.startWith.h5}")
    private String msgKyH5;

    @Value("${msg.ky.startWith.app}")
    private String msgKyApp;


    @Value("${msg.hth.startWith.web}")
    private String msgHthWeb;
    @Value("${msg.hth.startWith.h5}")
    private String msgHthH5;

    @Value("${msg.hth.startWith.app1}")
    private String msgHthApp1;

    @Value("${msg.hth.startWith.app2}")
    private String msgHthApp2;

    //js修改部分
    @Value("${js.ky.kyweb1}")
    private String jsKyWeb1;

    @Value("${js.ky.kyweb2}")
    private String jsKyWeb2;

    @Value("${js.ky.kyh51}")
    private String jsKyH51;

    @Value("${js.ky.kyh52}")
    private String jsKyH52;

    @Value("${js.ky.kyApp1}")
    private String jsKyApp1;

    @Value("${js.ky.kyApp2}")
    private String jsKyApp2;

    @Value("${js.hth.hthweb}")
    private String jsHthWeb;

    @Value("${js.hth.hthh5}")
    private String jsHthH5;

    @Value("${js.hth.hthApp1}")
    private String jsHthApp1;

    @Value("${js.hth.hthApp2}")
    private String jsHthApp2;

    @Override
    public Boolean change(String text) {
        //如果传入参数为空,返回修改失败
        if (StrUtil.isEmpty(text)) {
            return false;
        } else {
            text = StrUtil.subBefore(text, "SEO防拦截域名", false);
        }

        //处理传入参数
        String[] strings = StrUtil.splitToArray(text, SystemUtil.LINE_SEPARATOR);

        List<String> msgKyWebList = getByStartWith(strings, msgKyWeb, URL_PATTERN, PREFER);
        List<String> msgKyH5List = getByStartWith(strings, msgKyH5, URL_PATTERN, PREFER);
        List<String> msgKyAppList = getByStartWith(strings, msgKyApp, URL_PATTERN, PREFER);
        String template = loadTemplate(templatePath);
        List<Msg2JSEntity> listEntity = toListEntity(msgKyWebList, msgKyH5List, msgKyAppList);
        for (Msg2JSEntity msg2JSEntity : listEntity) {
            template = template.replace(msg2JSEntity.original(), msg2JSEntity.target());
        }
        for (String target : targets) {
            if (FileUtil.exist(target)) {
                FileUtil.del(target);
            } else {
                FileUtil.newFile(target);
            }
            FileUtil.writeString(template, target, "UTF-8");
        }

        return true;
    }

    /**
     * 从模板文件中读取文件
     *
     * @param path 文件路径
     * @return js文件模板中行List
     */
    @Cacheable("template")
    public String loadTemplate(String path) {
        if (FileUtil.exist(path)) {
            return FileUtil.readUtf8String(path);
        } else {
            throw new RuntimeException("模板文件不存在");
        }
    }

    /**
     * 处理传入飞机信息
     *
     * @param arr       字符数组
     * @param startWith 开始标识
     * @param pattern   url正则字符
     * @param prefer    优先
     * @return List
     */
    public List<String> getByStartWith(String[] arr, String startWith, String pattern, String prefer) {
        List<String> list = Arrays.stream(arr)
                .filter(item -> item.startsWith(startWith))
                .map(x -> ReUtil.getGroup1(pattern, x))
                .sorted(Comparator.comparing(str -> str.contains(prefer)))
                .collect(Collectors.toList());
        if (list.size() == 0) {
            throw new RuntimeException("飞机信息处理错误");
        }
        return list;
    }

    public List<Msg2JSEntity> toListEntity(List<String> msgKyWebList,
                                           List<String> msgKyH5List,
                                           List<String> msgKyAppList) {
        ArrayList<Msg2JSEntity> msg2JSEntities = new ArrayList<>();
        msg2JSEntities.add(build(jsKyWeb1, msgKyWebList.get(0)));
        msg2JSEntities.add(build(jsKyWeb2, msgKyWebList.get(1)));
        msg2JSEntities.add(build(jsKyH51, msgKyH5List.get(0)));
        msg2JSEntities.add(build(jsKyH52, msgKyH5List.get(1)));
        msg2JSEntities.add(build(jsKyApp1, msgKyAppList.get(0)));
        msg2JSEntities.add(build(jsKyApp2, msgKyAppList.get(1)));
        return msg2JSEntities.stream().filter(Objects::nonNull).collect(Collectors.toList());
    }

    public static Msg2JSEntity build(String js, String msg) {
        if (StrUtil.isNotEmpty(js) && StrUtil.isNotEmpty(msg)) {
            return new Msg2JSEntity("{{" + js + "}}", msg);
        }
        return null;
    }

}
